 SUBROUTINE read_3d_binary

!------------------------------------------------------------------------------!
! Actual revisions:
! -----------------
! Restart file has to be re-opened on all PEs except PE0
! +call of user_read_restart_data,
! -dopr_time_count,
! hom_sum, volume_flow_area, volume_flow_initial moved to read_var_list,
! reading of old profil parameters (cross_..., dopr_crossindex, profile_***)
! removed
!
! Former revisions:
! -----------------
! $Id: read_3d_binary.f90 145 2008-01-09 08:17:38Z raasch $
!
! 102 2007-07-27 09:09:17Z raasch
! +uswst, uswst_m, vswst, vswst_m
!
! 96 2007-06-04 08:07:41Z raasch
! +rho_av, sa, sa_av, saswsb, saswst
!
! 73 2007-03-20 08:33:14Z raasch
! +precipitation_amount, precipitation_rate_av, rif_wall, u_m_l, u_m_r, etc.,
! z0_av
!
! 19 2007-02-23 04:53:48Z raasch
! +qswst, qswst_m, tswst, tswst_m
! 
! RCS Log replace by Id keyword, revision history cleaned up
!
! Revision 1.4  2006/08/04 15:02:32  raasch
! +iran, iran_part
!
! Revision 1.1  2004/04/30 12:47:27  raasch
! Initial revision
!
!
! Description:
! ------------
! Binary input of variables and arrays from restart file
!------------------------------------------------------------------------------!

    USE arrays_3d
    USE averaging
    USE cloud_parameters
    USE control_parameters
    USE cpulog
    USE indices
    USE interfaces
    USE particle_attributes
    USE pegrid
    USE profil_parameter
    USE random_function_mod
    USE statistics

    IMPLICIT NONE

    CHARACTER (LEN=10) ::  binary_version, version_on_file
    CHARACTER (LEN=20) ::  field_chr

    INTEGER ::  idum1, myid_on_file, numprocs_on_file, nxl_on_file, &
                nxr_on_file, nyn_on_file, nys_on_file, nzb_on_file, nzt_on_file

!
!-- Read data from previous model run. unit 13 already opened in parin
    CALL cpu_log( log_point_s(14), 'read_3d_binary', 'start' )

!
!-- Restart file has to be re-opened on all PEs except PE0
    CALL check_open( 13 )

!
!-- First compare the version numbers
    READ ( 13 )  version_on_file
    binary_version = '3.1'
    IF ( TRIM( version_on_file ) /= TRIM( binary_version ) )  THEN
       IF ( myid == 0 )  THEN
          PRINT*, '+++ init_3d_model: version mismatch concerning data ', &
                  'from prior run'
          PRINT*, '        version on file    = "', TRIM( version_on_file ),&
                  '"'
          PRINT*, '        version in program = "', TRIM( binary_version ), &
                  '"'
       ENDIF
       CALL local_stop
    ENDIF

!
!-- Read and compare number of processors, processor-id and array ranges
    READ ( 13 )  numprocs_on_file, myid_on_file, nxl_on_file, nxr_on_file, &
                 nys_on_file, nyn_on_file, nzb_on_file, nzt_on_file

    IF ( numprocs_on_file /= numprocs )  THEN
       PRINT*, '+++ init_3d_model: mismatch between actual data and data '
       PRINT*, '                   from prior run on PE ', myid
       PRINT*, '                   numprocs on file = ', numprocs_on_file
       PRINT*, '                   numprocs         = ', numprocs
       CALL local_stop
    ENDIF

    IF ( myid_on_file /= myid )  THEN
       PRINT*, '+++ init_3d_model: mismatch between actual data and data '
       PRINT*, '                   from prior run'
       PRINT*, '                   myid_on_file = ', myid_on_file
       PRINT*, '                   myid         = ', myid
#if defined( __parallel )
       CALL MPI_ABORT( comm2d, 9999, ierr )
#else
       CALL local_stop
#endif
    ENDIF

    IF ( nxl_on_file /= nxl )  THEN
       PRINT*, '+++ init_3d_model: mismatch between actual data and data '
       PRINT*, '                   from prior run on PE ', myid
       PRINT*, '                   nxl on file = ', nxl_on_file
       PRINT*, '                   nxl         = ', nxl
#if defined( __parallel )
       CALL MPI_ABORT( comm2d, 9999, ierr )
#else
       CALL local_stop
#endif
    ENDIF

    IF ( nxr_on_file /= nxr )  THEN
       PRINT*, '+++ init_3d_model: mismatch between actual data and data '
       PRINT*, '                   from prior run on PE ', myid
       PRINT*, '                   nxr on file = ', nxr_on_file
       PRINT*, '                   nxr         = ', nxr
#if defined( __parallel )
       CALL MPI_ABORT( comm2d, 9999, ierr )
#else
       CALL local_stop
#endif
    ENDIF

    IF ( nys_on_file /= nys )  THEN
       PRINT*, '+++ init_3d_model: mismatch between actual data and data '
       PRINT*, '                   from prior run on PE ', myid
       PRINT*, '                   nys on file = ', nys_on_file
       PRINT*, '                   nys         = ', nys
#if defined( __parallel )
       CALL MPI_ABORT( comm2d, 9999, ierr )
#else
       CALL local_stop
#endif
    ENDIF

    IF ( nyn_on_file /= nyn )  THEN
       PRINT*, '+++ init_3d_model: mismatch between actual data and data '
       PRINT*, '                   from prior run on PE ', myid
       PRINT*, '                   nyn on file = ', nyn_on_file
       PRINT*, '                   nyn         = ', nyn
#if defined( __parallel )
       CALL MPI_ABORT( comm2d, 9999, ierr )
#else
       CALL local_stop
#endif
    ENDIF

    IF ( nzb_on_file /= nzb )  THEN
       PRINT*, '+++ init_3d_model: mismatch between actual data and data '
       PRINT*, '                   from prior run on PE ', myid
       PRINT*, '                   nzb on file = ', nzb_on_file
       PRINT*, '                   nzb         = ', nzb
       CALL local_stop
    ENDIF

    IF ( nzt_on_file /= nzt )  THEN
       PRINT*, '+++ init_3d_model: mismatch between actual data and data '
       PRINT*, '                   from prior run on PE ', myid
       PRINT*, '                   nzt on file = ', nzt_on_file
       PRINT*, '                   nzt         = ', nzt
       CALL local_stop
    ENDIF

!
!-- Initialize spectra (for the case of just starting spectra computation
!-- in continuation runs)
    IF ( dt_dosp /= 9999999.9 )  THEN
       spectrum_x = 0.0
       spectrum_y = 0.0
    ENDIF

!
!-- Read arrays
!-- ATTENTION: If the following read commands have been altered, the 
!-- ---------- version number of the variable binary_version must be altered,
!--            too. Furthermore, the output list of arrays in write_3d_binary
!--            must also be altered accordingly.
    READ ( 13 )  field_chr
    DO  WHILE ( TRIM( field_chr ) /= '*** end ***' )

       SELECT CASE ( TRIM( field_chr ) )

          CASE ( 'e' )
             READ ( 13 )  e
          CASE ( 'e_av' )
             ALLOCATE( e_av(nzb:nzt+1,nys-1:nyn+1,nxl-1:nxr+1) )
             READ ( 13 )  e_av
          CASE ( 'e_m' )
             READ ( 13 )  e_m
          CASE ( 'iran' )
             READ ( 13 )  iran, iran_part
          CASE ( 'kh' )
             READ ( 13 )  kh
          CASE ( 'kh_m' )
             READ ( 13 )  kh_m
          CASE ( 'km' )
             READ ( 13 )  km
          CASE ( 'km_m' )
             READ ( 13 )  km_m
          CASE ( 'lwp_av' )
             ALLOCATE( lwp_av(nys-1:nyn+1,nxl-1:nxr+1) )
             READ ( 13 )  lwp_av
          CASE ( 'p' )
             READ ( 13 )  p
          CASE ( 'p_av' )
             ALLOCATE( p_av(nzb:nzt+1,nys-1:nyn+1,nxl-1:nxr+1) )
             READ ( 13 )  p_av
          CASE ( 'pc_av' )
             ALLOCATE( pc_av(nzb:nzt+1,nys-1:nyn+1,nxl-1:nxr+1) )
             READ ( 13 )  pc_av
          CASE ( 'pr_av' )
             ALLOCATE( pr_av(nzb:nzt+1,nys-1:nyn+1,nxl-1:nxr+1) )
             READ ( 13 )  pr_av
          CASE ( 'precipitation_amount' )
             READ ( 13 )  precipitation_amount
          CASE ( 'precipitation_rate_a' )
             ALLOCATE( precipitation_rate_av(nys-1:nyn+1,nxl-1:nxr+1) )
             READ ( 13 )  precipitation_rate_av
          CASE ( 'pt' )
             READ ( 13 )  pt
          CASE ( 'pt_av' )
             ALLOCATE( pt_av(nzb:nzt+1,nys-1:nyn+1,nxl-1:nxr+1) )
             READ ( 13 )  pt_av
          CASE ( 'pt_m' )
             READ ( 13 )  pt_m
          CASE ( 'q' )
             READ ( 13 )  q
          CASE ( 'q_av' )
             ALLOCATE( q_av(nzb:nzt+1,nys-1:nyn+1,nxl-1:nxr+1) )
             READ ( 13 )  q_av
          CASE ( 'q_m' )
             READ ( 13 )  q_m
          CASE ( 'ql' )
             READ ( 13 )  ql
          CASE ( 'ql_av' )
             ALLOCATE( ql_av(nzb:nzt+1,nys-1:nyn+1,nxl-1:nxr+1) )
             READ ( 13 )  ql_av
          CASE ( 'ql_c_av' )
             ALLOCATE( ql_c_av(nzb:nzt+1,nys-1:nyn+1,nxl-1:nxr+1) )
             READ ( 13 )  ql_c_av
          CASE ( 'ql_v_av' )
             ALLOCATE( ql_v_av(nzb:nzt+1,nys-1:nyn+1,nxl-1:nxr+1) )
             READ ( 13 )  ql_v_av
          CASE ( 'ql_vp_av' )
             ALLOCATE( ql_vp_av(nzb:nzt+1,nys-1:nyn+1,nxl-1:nxr+1) )
             READ ( 13 )  ql_vp_av
          CASE ( 'qs' )
             READ ( 13 )  qs
          CASE ( 'qsws' )
             READ ( 13 )  qsws
          CASE ( 'qsws_m' )
             READ ( 13 )  qsws_m
          CASE ( 'qswst' )
             READ ( 13 )  qswst
          CASE ( 'qswst_m' )
             READ ( 13 )  qswst_m
          CASE ( 'qv_av' )
             ALLOCATE( qv_av(nzb:nzt+1,nys-1:nyn+1,nxl-1:nxr+1) )
             READ ( 13 )  qv_av
          CASE ( 'random_iv' )
             READ ( 13 )  random_iv
             READ ( 13 )  random_iy
          CASE ( 'rho_av' )
             ALLOCATE( rho_av(nzb:nzt+1,nys-1:nyn+1,nxl-1:nxr+1) )
             READ ( 13 )  rho_av
          CASE ( 'rif' )
             READ ( 13 )  rif
          CASE ( 'rif_m' )
             READ ( 13 )  rif_m
          CASE ( 'rif_wall' )
             READ ( 13 )  rif_wall
          CASE ( 's_av' )
             ALLOCATE( s_av(nzb:nzt+1,nys-1:nyn+1,nxl-1:nxr+1) )
             READ ( 13 )  s_av
          CASE ( 'sa' )
             READ ( 13 )  sa
          CASE ( 'sa_av' )
             ALLOCATE( sa_av(nzb:nzt+1,nys-1:nyn+1,nxl-1:nxr+1) )
             READ ( 13 )  sa_av
          CASE ( 'saswsb' )
             READ ( 13 )  saswsb
          CASE ( 'saswst' )
             READ ( 13 )  saswst
          CASE ( 'shf' )
             READ ( 13 )  shf
          CASE ( 'shf_m' )
             READ ( 13 )  shf_m
          CASE ( 'spectrum_x' )
             READ ( 13 )  spectrum_x
          CASE ( 'spectrum_y' )
             READ ( 13 )  spectrum_y
          CASE ( 'ts' )
             READ ( 13 )  ts
          CASE ( 'ts_av' )
             ALLOCATE( ts_av(nys-1:nyn+1,nxl-1:nxr+1) )
             READ ( 13 )  ts_av
          CASE ( 'tswst' )
             READ ( 13 )  tswst
          CASE ( 'tswst_m' )
             READ ( 13 )  tswst_m
          CASE ( 'u' )
             READ ( 13 )  u
          CASE ( 'u_av' )
             ALLOCATE( u_av(nzb:nzt+1,nys-1:nyn+1,nxl-1:nxr+1) )
             READ ( 13 )  u_av
          CASE ( 'u_m' )
             READ ( 13 )  u_m
          CASE ( 'u_m_l' )
             READ ( 13 )  u_m_l
          CASE ( 'u_m_n' )
             READ ( 13 )  u_m_n
          CASE ( 'u_m_r' )
             READ ( 13 )  u_m_r
          CASE ( 'u_m_s' )
             READ ( 13 )  u_m_s
          CASE ( 'us' )
             READ ( 13 )  us
          CASE ( 'usws' )
             READ ( 13 )  usws
          CASE ( 'uswst' )
             READ ( 13 )  uswst
          CASE ( 'usws_m' )
             READ ( 13 )  usws_m
          CASE ( 'uswst_m' )
             READ ( 13 )  uswst_m
          CASE ( 'us_av' )
             ALLOCATE( us_av(nys-1:nyn+1,nxl-1:nxr+1) )
             READ ( 13 )  us_av
          CASE ( 'v' )
             READ ( 13 )  v
          CASE ( 'v_av' )
             ALLOCATE( v_av(nzb:nzt+1,nys-1:nyn+1,nxl-1:nxr+1) )
             READ ( 13 )  v_av
          CASE ( 'v_m' )
             READ (13 )   v_m
          CASE ( 'v_m_l' )
             READ ( 13 )  v_m_l
          CASE ( 'v_m_n' )
             READ ( 13 )  v_m_n
          CASE ( 'v_m_r' )
             READ ( 13 )  v_m_r
          CASE ( 'v_m_s' )
             READ ( 13 )  v_m_s
          CASE ( 'vpt' )
             READ ( 13 )  vpt
          CASE ( 'vpt_av' )
             ALLOCATE( vpt_av(nzb:nzt+1,nys-1:nyn+1,nxl-1:nxr+1) )
             READ ( 13 )  vpt_av
          CASE ( 'vpt_m' )
             READ ( 13 )  vpt_m
          CASE ( 'vsws' )
             READ ( 13 )  vsws
          CASE ( 'vswst' )
             READ ( 13 )  vswst
          CASE ( 'vsws_m' )
             READ ( 13 )  vsws_m
          CASE ( 'vswst_m' )
             READ ( 13 )  vswst_m
          CASE ( 'w' )
             READ ( 13 )  w
          CASE ( 'w_av' )
             ALLOCATE( w_av(nzb:nzt+1,nys-1:nyn+1,nxl-1:nxr+1) )
             READ ( 13 )  w_av
          CASE ( 'w_m' )
             READ ( 13 )  w_m
          CASE ( 'w_m_l' )
             READ ( 13 )  w_m_l
          CASE ( 'w_m_n' )
             READ ( 13 )  w_m_n
          CASE ( 'w_m_r' )
             READ ( 13 )  w_m_r
          CASE ( 'w_m_s' )
             READ ( 13 )  w_m_s
          CASE ( 'z0' )
             READ ( 13 )  z0
          CASE ( 'z0_av' )
             ALLOCATE( z0_av(nys-1:nyn+1,nxl-1:nxr+1) )
             READ ( 13 )  z0_av

          CASE DEFAULT
             PRINT*, '+++ init_3d_model: unknown field named "', &
                     TRIM( field_chr ), '" found in'
             PRINT*, '                   data from prior run on PE ', myid
             CALL local_stop

       END SELECT
!
!--    Read next character string
       READ ( 13 )  field_chr

    ENDDO

!
!-- Read user-defined restart data
    CALL user_read_restart_data

!
!-- End of time measuring for reading binary data
    CALL cpu_log( log_point_s(14), 'read_3d_binary', 'stop' )

 END SUBROUTINE read_3d_binary
